import matplotlib.pyplot as plt
import numpy as np

def cum_rms(ff, asd):
    psd = asd**2
    ff2 = np.roll(ff,-1)
    ff2[-1] = ff2[-2]
    df = ff2-ff
    ms = np.cumsum((psd*df)[::-1], axis=0)[::-1]
    return np.sqrt(ms)

loaddict = {'unpack': 1, 'usecols': (0, 1), 'comments': '"'}
f_hg_ool, asd_hg_ool = np.loadtxt('RF02.TXT', **loaddict)[:,2:]
f_ifr_ool, asd_ifr_ool = np.loadtxt('RF03.TXT', **loaddict)[:,2:]

f_hg_rf, dbm_hg_rf = np.loadtxt('RF04.TXT', **loaddict)[:,2:]
f_ifr_rf, dbm_ifr_rf = np.loadtxt('RF06.TXT', **loaddict)[:,2:]
f_ocxo_rf, dbm_ocxo_rf = np.loadtxt('RF07.TXT', **loaddict)[:,2:]
f_hgb_rf, dbm_hgb_rf = np.loadtxt('RF08.TXT', **loaddict)[:,2:]

hg_bw = 30e3
ifr_bw = 30e3
ocxo_bw = 100
hgb_bw = 30e3

ool_cal = 0.0095 * 0.3

h_ool = plt.figure()
ax_ool = h_ool.add_subplot(111)
ax_ool.loglog(f_hg_ool, asd_hg_ool * ool_cal, 'b', label='HG in CER', alpha=0.6)
ax_ool.loglog(f_hg_ool, cum_rms(f_hg_ool, asd_hg_ool) * ool_cal, 'b--', alpha=0.6)
ax_ool.loglog(f_ifr_ool, asd_ifr_ool * ool_cal, 'r', label='IFR in CER', alpha=0.6)
ax_ool.loglog(f_ifr_ool, cum_rms(f_ifr_ool, asd_ifr_ool) * ool_cal, 'r--', alpha=0.6)
ax_ool.set_xlim(3e3, 1.05e6)
ax_ool.set_ylim(5e-10, 1.5e-4)
ax_ool.set_xlabel('Frequency [Hz]')
ax_ool.set_ylabel('ASD of RIN [1/Hz$^{1/2}$] or rms RIN')
ax_ool.set_title('Out-of-loop 45.5 MHz RIN after EOM driver')
ax_ool.legend(loc='best')
h_ool.tight_layout()
h_ool.savefig('ool_asd_45MHz.pdf')

h_rf = plt.figure()
ax_rf = h_rf.add_subplot(111)
ax_rf.plot(f_hg_rf/1e6, (dbm_hg_rf-10)-10*np.log10(hg_bw), 'b', label=r'45.5 MHz, HG in CER', alpha=0.6)
ax_rf.plot(f_ifr_rf/1e6, (dbm_ifr_rf-10)-10*np.log10(ifr_bw), 'r', label=r'45.5 MHz, IFR in CER', alpha=0.6)
#ax_rf.plot(f_ifr_rf/1e6, dbm_ifr_rf-10, 'r', label='45.5 MHz, IFR in CER', alpha=0.6)
ax_rf.plot(f_ocxo_rf/1e6*5, (dbm_ocxo_rf-8)-10*np.log10(5*ocxo_bw), 'g', label=r'9.1 MHz, OCXO in CER (shown$\times$5)', alpha=0.6)
ax_rf.plot(f_hgb_rf/1e6, (dbm_hgb_rf-10)-10*np.log10(hgb_bw), 'm', label=r'45.5 MHz, spare HG on bench', alpha=0.6)
ax_rf.set_xlabel('Frequency [MHz]')
ax_rf.set_ylabel('RIN [dBc/Hz]')
ax_rf.set_xticks(np.arange(40, 50.5, 1))
ax_rf.set_yticks(np.arange(-140, 11, 10))
ax_rf.set_xlim(45.5-4.6, 45.5+4.6)
ax_rf.set_ylim(-133, -28)
ax_rf.set_title('RF oscillator spectra, 2015-08-27')
ax_rf.legend(loc='best', prop={'size': 10})
h_rf.tight_layout()
h_rf.savefig('rf_asd_45MHz.pdf')
